// interface.cpp : Defines the exported functions for the DLL application.
//

#include <stdio.h>
#include "interface.h"
#include <math.h>
#include <string.h>
#include "rs232.h"

#define INTERFACE_RS232_DEBUG
#ifdef INTERFACE_RS232_DEBUG
#define PRINT printf
#else
#define PRINT
#endif


// This is an example of an exported function.

SYSTEM_SETTING_TYPE data0;
SYSTEM_SETTING_STRING data1;
SYSTEM_SETTING_COEF data2, data6;
SYSTEM_SETTING_NAME data3;
SYSTEM_SETTING_TCP data4;
SYSTEM_LINEARITY_COEF data5;

void print_config_string(int len, unsigned int *data);
int calcrc(unsigned char *data, int len);
int calcrc(unsigned char *data, int len)
{
    unsigned short crc=0xFFFF;
    int i;
    for (i=0; i<len; i++)
    {
        crc = (crc >> 8) | (crc << 8);
        crc ^= data[i];
        crc ^= (unsigned char)(crc & 0xff) >> 4;
        crc ^= (crc << 8) << 4;
        crc ^= ((crc & 0xff) << 4) << 1;
    }
    data[i++]=crc>>8;
    data[i++]=(unsigned char) crc&0xff;
//	printf("crc %x %x %x\n", crc);
    return(i);
}
unsigned short calcrc1(unsigned char *data, int len);
unsigned short  calcrc1(unsigned char *data, int len)
{
    unsigned short crc=0xFFFF;
    int i;
    for (i=0; i<len; i++)
    {
        crc = (crc >> 8) | (crc << 8);
        crc ^= data[i];
        crc ^= (unsigned char)(crc & 0xff) >> 4;
        crc ^= (crc << 8) << 4;
        crc ^= ((crc & 0xff) << 4) << 1;
    }
    return(crc);
}
int scandata(unsigned char *data, int len);
// 92 is backlash \ , 13 and 92 are key words.
int scandata(unsigned char *data, int len)
{
    int i, size, newlen, len1;
    size=len-2;
    newlen=len;
    for(i=0; i<size; i++)
    {
        if(data[i]==92 || data[i]==13)
        {
            newlen++;
            for(len1=newlen; len1>i; len1--)
                data[len1]=data[len1-1];
            data[i]=92;
            i++;
            size++;
        }
    }
    return(newlen);
}

void print_config_string(int len, unsigned int *data)
{
    char *ptr;
    int length;
    length=len*4;
    ptr=(char *)data;
    for(int i=0; i<length; i++)
    {
        if(ptr[i])
            printf("%c", ptr[i]);
        else
        {
            printf("\n");
            return;
        }
    }
    printf("\n");
}

INTERFACE_API double XSPEC_get_temperature(unsigned int bits)
{
    unsigned int bits1, bits2;
    double temperature;
    bits2=bits;
    temperature=bits & 0xFFFF;
    bits1=bits>>31;
    bits=0x7FFF & (bits>>16);
    if(bits1)
    {
        unsigned int temp1, temp2;
        temp1=bits&0xff;
        temp2=bits2>>24;
        if(temp1==16 && temp2==0x80)
        {
            //adt7410
            if(bits2 & 0x8000)
                return((temperature-65536)/128.0);
            else
                return((temperature)/128.0);
        }
        else
        {
            double max;
            double volt;
            max=pow(2.0,(int)bits&0xff);
            volt=bits>>8;
            volt=volt/10;
            temperature=(double)temperature/max*volt;
            temperature=((temperature)-0.8)*1000.0/2.65+27.0;
        }
    }
    else
    {
        if(bits==0)
        {
            temperature=temperature/1023.0*3.3;
            temperature=((temperature/5.7)-0.3)*1000.0+25.0;
        }
        else
        {
            double max;
            double volt;
            max=pow(2.0,(int)bits&0xff);
            volt=bits>>8;
            volt=volt/10;
            temperature=(double)temperature/max*volt;
            temperature=((temperature/5.7)-0.3)*1000.0+25.0;
        }
    }
    return(temperature);
}

void print_config()
{
    double temperature;//0.3v at 25c, 5.7X amplification 3.3=1023
    unsigned int bits, bits1;
    printf("- indicate not implemented\n");
    printf("integration time: %d\n", data0.integration_time);
    printf("integration mode: %d\n", data0.int_mode);
    printf("Trigger mode: %d\n", data0.trig_mode);
    printf("-extern_trigger_delay: %d\n", data0.extern_trigger_delay);
    printf("Skip # of reading: %d\n", data0.st_skip);
    printf("-Dark Offset: %d\n", data0.dark_offset);
    printf("LAMP PWN SETTING 0: 0x%X\n", data0.LAMP_PWM0);
    printf("LAMP PWM SETTING 1: 0x%X\n", data0.LAMP_PWM1);
    printf("Read # times: %d\n", data0.multiple_count);
    printf("LAMP Mode. 1==sync: 0x%X\n", data0.lamp_mode);
    printf("Port Direction 0==input, 1==output: 0x%X\n", data0.dir_port);
    printf("Port Reading:  0x%X\n", data0.port);


    temperature=XSPEC_get_temperature(data0.temp);
    printf("ADC temperature: %d %lf \n", data0.temp, temperature);
    printf("Flash Size:  0x%X\n", data0.flash_size);
    printf("Flash Page Size:  0x%X\n", data0.flash_page_size);
    printf("Flash setting_start:  0x%X\n", data0.flash_setting_start);
    printf("Flash start_auto_adj_dc_bias:  0x%X\n", data0.start_auto_adj_dc_bias);
    printf("Average:  0x%X\n", data0.average);

    printf("Number of Pixels: %d\n", data1.pixels);
    printf("Description:");
    print_config_string(4, data1.description);
    printf("Firmware Version:");
    print_config_string(4, data1.firmware_version);
    printf("Serial:");
    print_config_string(4, data1.serial);
    printf("Detector:");
    print_config_string(4, data1.detector);
    printf("Grating:");
    print_config_string(4, data1.grating);
    printf("Entrance Filter:");
    print_config_string(4, data1.entrance_filter);
    printf("Slit Size:");
    print_config_string(4, data1.slit_size);
    printf("Sorting Filter:");
    print_config_string(4, data1.sorting_filter);

    printf("Start Wavelength %lf\n", data2.wave_start);
    printf("End Wavelength %lf\n", data2.wave_end);
    printf("User Coef: \n");
    for(int i=0; i<6; i++)printf("\t%lf \n", data2.user_coef[i]);
    printf("Factory Coef: \n");
    for(int i=0; i<6; i++)printf("\t%lf \n", data2.system_coef[i]);
    printf("BIAS PWM SETTING: 0x%X\n", data2.BIAS_PWM);
    printf("Calibration Coef order: %d\n", data2.coef_order);
    printf("Dark Start Pixel: %d\n", data2.dark_start);
    printf("Dark End Pixel: %d\n", data2.dark_end);
    printf("Data Start Pixel: %d\n", data2.data_start);
    printf("Data End Pixel: %d\n", data2.data_end);

    printf("User String: ");
    print_config_string(16, data3.user_name);
    printf("Compiled Date String: ");
    print_config_string(8, data3.date_string);
    printf("Compiled Time String: ");
    print_config_string(8, data3.time_string);

    printf("Ethernet Settings: ");
    printf("mac: %2x:%2x:%2x:%2x:%2x:%2x\n", data4.mac[0], data4.mac[1], data4.mac[2], data4.mac[3], data4.mac[4], data4.mac[5]);
    printf("IP: %d.%d.%d.%d\n", data4.ip[0], data4.ip[1],data4.ip[2],data4.ip[3]);
    printf("Gateway: %d.%d.%d.%d\n", data4.gate[0], data4.gate[1],data4.gate[2],data4.gate[3]);
    printf("Mask: %d.%d.%d.%d\n", data4.mask[0], data4.mask[1],data4.mask[2],data4.mask[3]);
    printf("Tcp Port: %d\n", data4.tcp_port);
    printf("Http Port: %d\n", data4.http_port);
    printf("Use this setting or default: %d\n", data4.use_this);
    printf("default: 12.34.56.78.9a.bc, 100.0.0.100, 100.0.0.1, 255.255.255.0, 8888, 80 ");
}




//==============================================================================================================
//RS232

//open the XSPEC device by comm port number 0,1,2,3,4...
//return handle, NULL if error
INTERFACE_API int XSPEC_rs232_open(unsigned char portnr)
{
    int   bStatus;
    int bdrate=115200;
    char mode[]= {'8','N','1',0};
    if(RS232_OpenComport(portnr, bdrate, mode))
    {
        printf("Can not open comport\n");

        return(0);
    }

    return(portnr);
}
//call this to close the rs232. 0==success, < 0 on error.
INTERFACE_API int XSPEC_rs232_close(unsigned char portnr)
{
    RS232_CloseComport(portnr);
    return(0);
}



// set the integration time in usec for the current_devnum, stays in RAM, reverted th flash after reboot
// return the number of bytes sent for the command, must be larger than 0.
INTERFACE_API int XSPEC_rs232_setIntegrationTime(unsigned char portnr,unsigned int time_us)
{
    unsigned char buffer[20];
    int newlen;
    int   bStatus;
    int wrotelen;

    buffer[0]=RS_SET_ITIME_CMD;
    buffer[1]=(unsigned char) time_us>>24;
    buffer[2]=(unsigned char)((time_us>>16) & 0xff);
    buffer[3]=(unsigned char)((time_us>>8) & 0xff);
    buffer[4]=(unsigned char)(time_us & 0xff);
    newlen=calcrc(buffer, 5);

    buffer[newlen++]=13;
    buffer[newlen++]=13;
    buffer[newlen]=0;
    newlen=scandata(buffer, newlen);
    buffer[newlen]=0;

    bStatus = RS232_SendBuf(portnr, buffer, newlen);
    return(bStatus);
}

// set the integration time in usec for the current_devnum, stays in FLASH
// return the number of bytes sent for the command, must be larger than 0.
INTERFACE_API int XSPEC_rs232_setIntegrationTimeFlash(unsigned char portnr,unsigned int time_us)
{
    SYSTEM_SETTING_TYPE data0;
    int ret;
    ret=XSPEC_rs232_get_SpecConfig0123(portnr,(char *)&data0, 0);
    if(ret)
        return(ret);
    data0.integration_time=time_us;
    ret=XSPEC_rs232_write_SpecConfig(portnr, (char *)&data0, 0);
    return(ret);
}
// set the integration mode for the current_devnum, stays in FLASH
// mode 0: normal. software
// mode 1: rising external edge
// mode 2: falling external edge
// return the number of bytes sent for the command, must be larger than 0.
INTERFACE_API int XSPEC_rs232_setModeFlash(unsigned char portnr,unsigned int mode)
{
    SYSTEM_SETTING_TYPE data0;
    int ret;
    ret=XSPEC_rs232_get_SpecConfig0123(portnr,(char *)&data0, 0);
    if(ret)
        return(ret);
    data0.trig_mode=mode;
    ret=XSPEC_rs232_write_SpecConfig(portnr, (char *)&data0, 0);
    return(ret);
}
// set the firmware averaging for the current_devnum, stays in FLASH
// return the number of bytes sent for the command, must be larger than 0.
INTERFACE_API int XSPEC_rs232_setAverageFlash(unsigned char portnr,unsigned int average)
{
    SYSTEM_SETTING_TYPE data0;
    int ret;
    if(average<1)
        average=1;
    ret=XSPEC_rs232_get_SpecConfig0123(portnr,(char *)&data0, 0);
    if(ret)
        return(ret);
    data0.average=average;
    ret=XSPEC_rs232_write_SpecConfig(portnr, (char *)&data0, 0);
    return(ret);
}

//set sync mode for flash lamp, pass in 1 or 0, 1 to enable, 0 to disable. 
INTERFACE_API int XSPEC_rs232_Sync(unsigned char portnr,unsigned char val)
{
    SYSTEM_SETTING_TYPE data0;
    int ret;
    ret=XSPEC_rs232_get_SpecConfig0123(portnr,(char *)&data0, 0);
    if(ret)
        return(ret);
	if(val)
    	data0.lamp_mode|=1;
	else
		data0.lamp_mode&=(~1);
    ret=XSPEC_rs232_write_SpecConfig(portnr, (char *)&data0, 0);
    return(ret);
}


// 4 data structure stored in the flash. return one of them, use index (0,1,2,3) to indicate which one.
// create the data for p_data, it is 128bytes. use get_SpecConfig() to get the entire structure. Modify the one you need
// to change and put the data structure back using this function.
// return number of bytes sent, -1==failure,
INTERFACE_API int XSPEC_rs232_write_SpecConfig(unsigned char portnr,char *p_data, int index)
{
    int i, newlen;
    char *int_ptr;
    int bStatus;
    unsigned char write_buffer[FLASH_PAGE_SIZE*2];
    int page;
    int wrotelen;

    if(index<0 || index>3)
        return(-1);


    write_buffer[0]=RS_WRITE_NVMEM_CMD;
    page=index;
    i=page>>8;
    write_buffer[1]=(char) i;
    i=page & 0xFF;
    write_buffer[2]=(char)i;
    write_buffer[3]=1;   //not used. for alignment
    //src=(char *)&(p_data);
//	temp=(char *)temp_int;
//	for(i=0;i<FLASH_PAGE_SIZE;i++)
    //	temp[i]=src[i];

    int_ptr=(char  *)&(write_buffer[4]);

    for(i=0; i<FLASH_PAGE_SIZE; i++)
        int_ptr[i]=p_data[i];

    newlen=calcrc((unsigned char *)write_buffer, 132);
    write_buffer[newlen++]=13;
    write_buffer[newlen++]=13;
    write_buffer[newlen]=0;

    newlen=scandata((unsigned char *)write_buffer, newlen);
    write_buffer[newlen]=0;




    bStatus = RS232_SendBuf(portnr, write_buffer, newlen);
    return(bStatus);

}
//only update the setting0 in memory, not update the flash
INTERFACE_API int XSPEC_rs232_update_config_mem(unsigned char portnr,char *p_data)
{
    int i, newlen;
    char *int_ptr;
    unsigned char write_buffer[FLASH_PAGE_SIZE*2];
    int bStatus;
    int wrotelen;

    write_buffer[0]=RS_WRITE_SETTING_CMD;   //update memory only, not flash;
    write_buffer[1]=(char) 0;
    write_buffer[2]=(char)1;
    write_buffer[3]=1;   //not used. for alignment
//	temp=(char *)temp_int;
//	for(i=0;i<FLASH_PAGE_SIZE;i++)
//		temp[i]=src[i];

    int_ptr=(char *)&(write_buffer[4]);

    for(i=0; i<FLASH_PAGE_SIZE; i++)
        int_ptr[i]=p_data[i];

    newlen=calcrc((unsigned char *)write_buffer, 132);

    write_buffer[newlen++]=13;
    write_buffer[newlen++]=13;
    write_buffer[newlen]=0;

    newlen=scandata((unsigned char *)write_buffer, newlen);
    write_buffer[newlen]=0;




    bStatus = RS232_SendBuf(portnr, write_buffer, newlen);
    return(bStatus);
}




//retrieve the 4 data structures stored in the flash and save them locally.
// 0==success, -1==failure.
INTERFACE_API int XSPEC_rs232_get_SpecConfig(unsigned char portnr)
{
    unsigned char data_buffer[FLASH_PAGE_SIZE*10];
    int newlen, index,bytes_read;
    unsigned short *buffer, crc, *data, type, size;
    unsigned char write_buffer[10];
    int bStatus;
    int wrotelen;

    write_buffer[0]=RS_READ_NVMEM_CMD;
    newlen=calcrc(write_buffer, 1);
    write_buffer[newlen++]=13;
    write_buffer[newlen++]=13;
    write_buffer[newlen]=0;
    newlen=scandata((unsigned char *)write_buffer, newlen);
    write_buffer[newlen]=0;



    bStatus = RS232_SendBuf(portnr, write_buffer, newlen);
    PRINT("RS232_SendBuf bstatus=%d, writelen=%d\n", bStatus, newlen);
    fflush(stdout);
    if (bStatus != newlen)
        return(bStatus);

    bStatus = RS232_PollComport(portnr, data_buffer, 776);
    PRINT("RS232_PollComport bstatus=%d, want to read=%d \n", bStatus, 776);
    fflush(stdout);


    //for(int i=0;i<bytes_read;i++)
    // 	printf(" %x ", data_buffer[i]);
    //printf("\n");
    //fflush(stdout);
    if(bStatus!=776)
        return(-1);

    buffer=(unsigned short *) data_buffer;

    index=0;
    if(buffer[index++]!=PACKET_START)
    {
        printf("wrong start value %x\n", buffer[index-1]);
        return(-1);
    }

    type=buffer[index++];
    if(type==RS_READ_NVMEM_CMD)
    {
        SYSTEM_SETTING_TYPE setting0;
        SYSTEM_SETTING_STRING setting1;
        SYSTEM_SETTING_COEF setting2;
        SYSTEM_SETTING_NAME setting3;
        SYSTEM_SETTING_TCP setting4;
        SYSTEM_LINEARITY_COEF setting5;
        size=buffer[index++];
        data=(unsigned short *) &setting0; //&(node_ptr->setting0);
        for(int tmp=0; tmp<FLASH_PAGE_SIZE/2; tmp++)
        {
            data[tmp]=buffer[index++];
        }
        data=(unsigned short *) &setting1; //&(node_ptr->setting1);
        for(int tmp=0; tmp<FLASH_PAGE_SIZE/2; tmp++)
        {
            data[tmp]=buffer[index++];
        }
        data=(unsigned short *) &setting2; //&(node_ptr->setting2);
        for(int tmp=0; tmp<FLASH_PAGE_SIZE/2; tmp++)
        {
            data[tmp]=buffer[index++];
        }
        data=(unsigned short *) &setting3; //&(node_ptr->setting3);
        for(int tmp=0; tmp<FLASH_PAGE_SIZE/2; tmp++)
        {
            data[tmp]=buffer[index++];
        }
        data=(unsigned short *) &setting4; //&(node_ptr->setting3);
        for(int tmp=0; tmp<FLASH_PAGE_SIZE/2; tmp++)
        {
            data[tmp]=buffer[index++];
        }
        data=(unsigned short *) &setting5; //&(node_ptr->setting3);
        for(int tmp=0; tmp<FLASH_PAGE_SIZE/2; tmp++)
        {
            data[tmp]=buffer[index++];
        }
        crc=calcrc1(data_buffer, index*2);
        if(crc!=buffer[index++])
        {
            printf("wrong crc value %x %x\n", crc, buffer[index-1]);
            return (-1);
        }
        memcpy((char *)&(data0), (char *)&setting0, sizeof(SYSTEM_SETTING_TYPE));
        memcpy((char *)&(data1), (char *)&setting1, sizeof(SYSTEM_SETTING_STRING));
        memcpy((char *)&(data2), (char *)&setting2, sizeof(SYSTEM_SETTING_COEF));
        memcpy((char *)&(data3), (char *)&setting3, sizeof(SYSTEM_SETTING_NAME));
        memcpy((char *)&(data4), (char *)&setting4, sizeof(SYSTEM_SETTING_TCP));
        memcpy((char *)&(data5), (char *)&setting5, sizeof(SYSTEM_LINEARITY_COEF));
        return(0);
    }
    else
    {
        printf("wrong type  value %x\n", type);
        return(-1);
    }

}
// dump the  4 data structures using printf
// // 0==success, else==failure
INTERFACE_API int XSPEC_rs232_print_config(unsigned char portnr)
{
    int ret;
    ret=XSPEC_rs232_get_SpecConfig(portnr);
    if(ret)
    {
        printf("get_SpecConfig error\n");
        return(ret);
    }
    else
    {
        print_config();
    }
    return(0);
}
// pre-allocate BUF_SIZE for the data_array, id is an unique ID to distingusish multiple users accessing the same device.
// passed integ_time help determine how long to wait for the data. in usec; integ_time/1000+1000ms
// pass rs_start and rs_end (0-3647) for normal operation. SAM3U //36-3683
// return number of bytes received. <=0 indicates error.

INTERFACE_API int XSPEC_rs232_measure_partial(unsigned char portnr,unsigned short *data_array, unsigned int id, unsigned int integ_time, unsigned short rs_start, unsigned short rs_end)
{
    int bytesread=0, newlen;
    int bStatus,scount;
    int wrotelen, bytes_read;
    unsigned char write_buffer[FLASH_PAGE_SIZE];
    unsigned short crc, oldcrc;

    scount=(PIXELS-TCD1304_DATA_END-1)+(rs_end-rs_start+1);
    write_buffer[0]=RS_TRIG_CMD_PARTIAL;
    write_buffer[1]=(unsigned char)id;
    id=id>>8;
    write_buffer[2]=(unsigned char)id;
    id=id>>8;
    write_buffer[3]=(unsigned char)id;
    id=id>>8;
    write_buffer[4]=(unsigned char)id;
    write_buffer[5]=(unsigned char)rs_start;
    rs_start=rs_start>>8;
    write_buffer[6]=(unsigned char)rs_start;
    write_buffer[7]=(unsigned char)rs_end;
    rs_end=rs_end>>8;
    write_buffer[8]=(unsigned char)rs_end;


    newlen=calcrc(write_buffer, 9);
    write_buffer[newlen++]=13;
    write_buffer[newlen++]=13;
    write_buffer[newlen]=0;

    newlen=scandata(write_buffer, newlen);
    write_buffer[newlen]=0;

    bStatus = RS232_SendBuf(portnr, write_buffer, newlen);
    //	printf("bstatus=%d, wrotelen=%d\n", bStatus, wrotelen);fflush(stdout);
    if (bStatus != newlen)
        return(bStatus);

    // for 3A4C system, it is scount*2+6
    bStatus = RS232_PollComport(portnr, (unsigned char *)data_array, scount*2+2);
    if(bStatus!=scount*2+62)
    {
        printf("RS232_PollComport asked %d got %d\n", scount*2+2, bStatus);
        return(-1);
    }

//	printf("read back %d\n", bytes_read);fflush(stdout);
    crc=calcrc1((unsigned char *)data_array, scount*2);
    oldcrc=data_array[scount];

    if(crc!=oldcrc)
    {
        printf("crc not match %x %x\n", crc, oldcrc);
        return(-1);
    }
    if(bStatus==0)
        return(-1);

    return(bytesread);
}


// pre-allocate BUF_SIZE for the data_array, id is an unique ID to distingusish multiple users accessing the same device.
// passed integ_time help determine how long to wait for the data. in usec; integ_time/1000+1000ms
// return number of bytes received. <=0 indicates error.
INTERFACE_API int XSPEC_rs232_measure(unsigned char portnr,unsigned short *data_array, unsigned int id, unsigned int integ_time)
{
    int bytesread=0, newlen;
    int bStatus;
    int wrotelen, bytes_read;
    unsigned char write_buffer[FLASH_PAGE_SIZE];
    unsigned short crc, oldcrc;

    write_buffer[0]=RS_TRIG_CMD;
    write_buffer[1]=(unsigned char)id;
    id=id>>8;
    write_buffer[2]=(unsigned char)id;
    id=id>>8;
    write_buffer[3]=(unsigned char)id;
    id=id>>8;
    write_buffer[4]=(unsigned char)id;
    newlen=calcrc(write_buffer, 5);
    write_buffer[newlen++]=13;
    write_buffer[newlen++]=13;
    write_buffer[newlen]=0;

    newlen=scandata(write_buffer, newlen);
    write_buffer[newlen]=0;

    bStatus = RS232_SendBuf(portnr, write_buffer, newlen);
    //	printf("bstatus=%d, wrotelen=%d\n", bStatus, wrotelen);fflush(stdout);
    if (bStatus != newlen)
        return(bStatus);

    //  printf("wrotelen  %d\n", wrotelen);fflush(stdout);



    bStatus = RS232_PollComport(portnr,(unsigned char *)data_array, PIXELS*2+2);
    if(bStatus!=PIXELS*2+2)
        return(-1);

//	printf("read back %d\n", bytes_read);fflush(stdout);
    crc=calcrc1((unsigned char *)data_array, PIXELS*2);
    oldcrc=data_array[PIXELS];

    if(crc!=oldcrc)
    {
        printf("XSPEC_rs232_measure crc not match cal=%x rec=%x\n", crc, oldcrc);
        return(-1);
    }
    if(bStatus==0)
        return(-1);
//	printf("**measure one crc match cal=%x rec=%x\n", crc, oldcrc);
    return(bytesread);
}


INTERFACE_API int XSPEC_rs232_pwm_start(unsigned char portnr, int which)
{
    int bytesread=0, newlen;
    int wrotelen, bStatus;
    unsigned char write_buffer[FLASH_PAGE_SIZE];

    write_buffer[0]=RS_START_PWM_CMD;
    write_buffer[1]=which;
    newlen=calcrc(write_buffer, 2);
    write_buffer[newlen++]=13;
    write_buffer[newlen++]=13;
    write_buffer[newlen]=0;

    newlen=scandata(write_buffer, newlen);
    write_buffer[newlen]=0;


    bStatus = RS232_SendBuf(portnr, write_buffer, newlen);
    return(bStatus);

}

INTERFACE_API int XSPEC_rs232_pwm_stop(unsigned char portnr, int which)
{
    int bytesread=0, newlen;
    unsigned char write_buffer[FLASH_PAGE_SIZE];
    int wrotelen, bStatus;

    write_buffer[0]=RS_END_PWM_CMD;
    write_buffer[1]=which;
    newlen=calcrc(write_buffer, 2);
    write_buffer[newlen++]=13;
    write_buffer[newlen++]=13;
    write_buffer[newlen]=0;

    newlen=scandata(write_buffer, newlen);
    write_buffer[newlen]=0;

    bStatus = RS232_SendBuf(portnr, write_buffer, newlen);
    return(bStatus);

}
// stop transmission,
INTERFACE_API int XSPEC_rs232_abort(unsigned char portnr)
{
    int bytesread=0, newlen;
    int wrotelen, bStatus;
    unsigned char write_buffer[FLASH_PAGE_SIZE];

    write_buffer[0]=RS_ABORT_CMD;
    newlen=calcrc(write_buffer, 1);
    write_buffer[newlen++]=13;
    write_buffer[newlen++]=13;
    write_buffer[newlen]=0;

    newlen=scandata(write_buffer, newlen);
    write_buffer[newlen]=0;

    bStatus = RS232_SendBuf(portnr, write_buffer, newlen);
    return(bStatus);

}
// Restart the device. goto address "0"
INTERFACE_API int XSPEC_rs232_reset(unsigned char portnr)
{
    int bytesread=0, newlen;
    int wrotelen, bStatus;
    unsigned char write_buffer[FLASH_PAGE_SIZE];

    write_buffer[0]=RS_USB_RESET;
    newlen=calcrc(write_buffer, 1);
    write_buffer[newlen++]=13;
    write_buffer[newlen++]=13;
    write_buffer[newlen]=0;

    newlen=scandata(write_buffer, newlen);
    write_buffer[newlen]=0;

    bStatus = RS232_SendBuf(portnr, write_buffer, newlen);
    return(bStatus);

}
INTERFACE_API int XSPEC_rs_measure_stop(unsigned char portnr)
{
    int bytesread=0, newlen;
    int wrotelen;
    int bStatus;
    unsigned char write_buffer[FLASH_PAGE_SIZE];

    write_buffer[0]=RS_ABORT_CMD;
    newlen=calcrc(write_buffer, 1);
    write_buffer[newlen++]=13;
    write_buffer[newlen++]=13;
    write_buffer[newlen]=0;

    newlen=scandata(write_buffer, newlen);
    write_buffer[newlen]=0;

    bStatus = RS232_SendBuf(portnr, write_buffer, newlen);
    return(bStatus);

}
// return the integration time in usec for the current_devnum
// -1==failure,
INTERFACE_API int XSPEC_rs232_getIntegrationTime(unsigned char portnr)
{
    int ret;
    ret=XSPEC_rs232_get_SpecConfig(portnr);
    if(ret)
        return(ret);
    else
    {
        return(data0.integration_time);
    }
}

// return the firmware average number for the current_devnum
// -1==failure,
INTERFACE_API int XSPEC_rs232_getFirmwareAverage(unsigned char portnr)
{
    int ret;
    ret=XSPEC_rs232_get_SpecConfig(portnr);
    if(ret)
        return(ret);
    else
    {
        return(data0.average);
    }
}
// return the trigger mode for the current_devnum
// -1==failure,
INTERFACE_API int XSPEC_rs232_getMode(unsigned char portnr)
{
    int ret;
    ret=XSPEC_rs232_get_SpecConfig(portnr);
    if(ret)
        return(ret);
    else
    {
        return(data0.trig_mode);
    }
}

INTERFACE_API int XSPEC_rs232_upgrade(unsigned char portnr)
{
    int bytesread=0, newlen;
    int wrotelen;
    int bStatus;
    unsigned char write_buffer[FLASH_PAGE_SIZE];

    write_buffer[0]=UPGRADE_CMD;
    newlen=calcrc(write_buffer, 1);
    write_buffer[newlen++]=13;
    write_buffer[newlen++]=13;
    write_buffer[newlen]=0;

    newlen=scandata(write_buffer, newlen);
    write_buffer[newlen]=0;

    bStatus = RS232_SendBuf(portnr, write_buffer, newlen);
    return(bStatus);

}
INTERFACE_API int XSPEC_rs232_get_SpecProg(unsigned char portnr, unsigned short page)
{
    int bytesread=0, newlen;
    int wrotelen;
    int bStatus;
    unsigned char write_buffer[FLASH_PAGE_SIZE];

    write_buffer[0]=RS_READ_NVMEM_PRG;
    write_buffer[1]=page>>8;
    write_buffer[2]=page&0xff;;
    newlen=calcrc(write_buffer, 3);
    write_buffer[newlen++]=13;
    write_buffer[newlen++]=13;
    write_buffer[newlen]=0;

    newlen=scandata(write_buffer, newlen);
    write_buffer[newlen]=0;

    bStatus = RS232_SendBuf(portnr, write_buffer, newlen);
    return(bStatus);

}

// 4 data structure stored in the flash. return one of them, use index (0,1,2,3) to indicate which one.
// pre-allocate the space for p_data, it is 128bytes. cast to the structure defined in myusb.h
// -1==failure, 0==success
INTERFACE_API int XSPEC_rs232_get_SpecConfig0123(unsigned char portnr,char *p_data, int index)
{
    int ret;
    char *ptr;

    ret=XSPEC_rs232_get_SpecConfig(portnr);
    if(ret)
        return(ret);
    if(index==0)
        ptr=(char *)&data0;
    else if(index==1)
        ptr=(char *)&data1;
    else if(index==2)
        ptr=(char *)&data2;
    else if(index==3)
        ptr=(char *)&data3;
    else if(index==4)
        ptr=(char *)&data4;
    memcpy((char *)p_data, (char *)ptr, FLASH_PAGE_SIZE);
    return(0);
}


