// xspec_control.cpp : Defines the entry point for the console application.
//

#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "interface.h"

bool verbose;
void PRINT(const char *format, ...);
void printusage();
void process_data(int avg);
SYSTEM_SETTING_COEF cal_coef;
unsigned short data_array[4000];
double data_avg[4000];
int average;
void print_data();
unsigned short rs_start=0, rs_end=3647;

int main(int argc, char* argv[])
{
	int c;
	int comport;
	int devnum;
	int int_time;
	int devices=0;
	bool dump_config_one=false;
	bool get_int=false;
	bool set_int=false;
	bool set_int_flash=false;	
	bool measure_one=false;
	bool measure_partial=false;
	bool rs_abort=false;
	int mode=0, firmAveraging=1;
	bool setMode=false;
	bool setAvg=false;

	average=1;
	
	verbose=true;

	devnum=-1;
	if(argc<2)
	{
		printusage();
		return(0);
	}
	while ((c = getopt(argc, argv, ("mcbhipd:I:L:n:M:a:s:e:"))) != EOF)
	{			
		switch (c)
		{
			case ('b'):
				rs_abort=true;
				break;
			case ('h'):
				printusage();
				break;
			case ('d'):
				devnum=atoi(optarg);
				break;	
			case ('s'):
				rs_start=atoi(optarg);
				if(rs_start<0)rs_start=0;
				break;	
			case ('e'):
				rs_end=atoi(optarg);
				if(rs_end>3647)rs_end=3647;
				break;	
			case ('M'):
				mode=atoi(optarg);
				setMode=true;
				break;	
			case ('a'):
				firmAveraging=atoi(optarg);
				setAvg=true;
				break;	
			case ('c'):
				dump_config_one=true;
				break;	
			case ('i'):
				get_int=true;
				break;	
			case ('I'):
				int_time=atoi(optarg);
				set_int=true;
				break;	
			case ('L'):
				int_time=atoi(optarg);
				set_int_flash=true;
				break;	
			case ('n'):
				average=atoi(optarg);
				break;	
			case ('m'):
				measure_one=true;get_int=true;
				break;	
			case ('p'):
				measure_partial=true;get_int=true;
				break;
			default:		// legal option - it was specified in optstring,
							// but it had no "case" handler 

				// Note: you may choose not to make this an error

				PRINT("WARNING:  no handler for option %c\n",c);
				return (-1);
				break;
		}
	}
	if(dump_config_one)
	{
		if(devnum<0)
		{
			PRINT("devnum %d not defined, use -d#  define a device\n", devnum);
			goto EXIT;
		}

		comport=XSPEC_rs232_open(devnum);
		if(comport==0)
		{
			PRINT("Cannot open devnum %d\n", devnum);
			goto EXIT;
		}
		XSPEC_rs232_print_config(devnum);
		XSPEC_rs232_close(devnum);
		dump_config_one=false;
	}
	if(rs_abort)
	{
		if(devnum<0)
		{
			PRINT("devnum %d not defined, use -d# to define a device\n", devnum);
			goto EXIT;
		}

		comport=XSPEC_rs232_open(devnum);
		if(comport==0)
		{
			PRINT("Cannot open devnum %d\n", devnum);
			goto EXIT;
		}
		int temp_int_time=XSPEC_rs232_abort(devnum);
		XSPEC_rs232_close(devnum);
		rs_abort=false;
	}
	if(get_int)
	{
		if(devnum<0)
		{
			PRINT("devnum %d not defined, use -d# to define a device\n", devnum);
			goto EXIT;
		}

		comport=XSPEC_rs232_open(devnum);
		if(comport==0)
		{
			PRINT("Cannot open devnum %d\n", devnum);
			goto EXIT;
		}
		int temp_int_time=XSPEC_rs232_getIntegrationTime(devnum);
		PRINT("Trigger mode for comport %d=%d\n", devnum, XSPEC_rs232_getMode(devnum));
		PRINT("Integration Time for comport %d=%d\n", devnum, temp_int_time);
		PRINT("Firmware Average for comport %d=%d\n", devnum, XSPEC_rs232_getFirmwareAverage(devnum));
		XSPEC_rs232_close(devnum);
		get_int=false;
	}
	if(set_int)
	{
		if(devnum<0)
		{
			PRINT("devnum %d not defined, use -d# to define a device\n", devnum);
			goto EXIT;
		}
		PRINT("set devnum %d , int_time=%d\n", devnum, int_time);
		comport=XSPEC_rs232_open(devnum);
		if(comport==0)
		{
			PRINT("Cannot open devnum %d\n", devnum);
			goto EXIT;
		}
		XSPEC_rs232_setIntegrationTime(devnum, int_time);
		XSPEC_rs232_close(devnum);
		set_int=false;
	}
	if(set_int_flash)
	{
		int ret;
		if(devnum<0)
		{
			PRINT("devnum %d not defined, use -d# to define a device\n", devnum);
			goto EXIT;
		}
		PRINT("set devnum %d , int_time=%d\n", devnum, int_time);
		comport=XSPEC_rs232_open(devnum);
		if(comport==0)
		{
			PRINT("Cannot open devnum %d\n", devnum);
			goto EXIT;
		}
		XSPEC_rs232_setIntegrationTimeFlash(devnum, int_time);
		XSPEC_rs232_close(devnum);
		set_int_flash=false;
	}
	if(setMode)
	{
		int ret;
		if(devnum<0)
		{
			PRINT("devnum %d not defined, use -d# to define a device\n", devnum);
			goto EXIT;
		}
		PRINT("set mode COM=%d , mode=%d\n", devnum, mode);
		if(mode<0 || mode>2)
		{
			PRINT("Mode is out of bound\n");
			goto EXIT;
		}
		comport=XSPEC_rs232_open(devnum);
		if(comport==0 )
		{
			PRINT("Cannot open devnum %d\n", devnum);
			goto EXIT;
		}

		XSPEC_rs232_setModeFlash(devnum, mode);
		XSPEC_rs232_close(devnum);
		setMode=false;
	}
	if(setAvg)
	{
		int ret;
		if(devnum<0)
		{
			PRINT("devnum %d not defined, use -d# to define a device\n", devnum);
			goto EXIT;
		}
		PRINT("set firmware averaging: COM=%d , mode=%d\n", devnum, firmAveraging);
		if(firmAveraging<1)
		{
			PRINT("Mode is out of bound\n");
			goto EXIT;
		}
		comport=XSPEC_rs232_open(devnum);
		if(comport==0 )
		{
			PRINT("Cannot open devnum %d\n", devnum);
			goto EXIT;
		}

		XSPEC_rs232_setAverageFlash(devnum, firmAveraging);
		XSPEC_rs232_close(devnum);
		setAvg=false;
	}
	if(measure_one)
	{

		if(devnum<0)
		{
			PRINT("devnum %d not defined, use -d# to define a device\n", devnum);
			goto EXIT;
		}

		PRINT("set devnum %d \n", devnum);
		comport=XSPEC_rs232_open(devnum);
		if(comport==0)
		{
			PRINT("Cannot open devnum %d\n", devnum);
			goto EXIT;
		}
		int temp_int_time=XSPEC_rs232_getIntegrationTime(devnum);
		PRINT("Integ time = %d\n",  temp_int_time);
		XSPEC_rs232_get_SpecConfig0123(devnum, (char *)&cal_coef, 2);
		PRINT("Got coeff \n");
		for(int j=0;j<average;j++)
		{
			XSPEC_rs232_measure(devnum, data_array, 0x1234, temp_int_time);
			PRINT("Read %d/%d\n", j, average);
			process_data(j);
		}
		XSPEC_rs232_close(devnum);
		rs_start=0;
		rs_end=3647;
		print_data();

		measure_one=false;
	}
	if(measure_partial)
	{

		if(devnum<0)
		{
			PRINT("devnum %d not defined, use -d# to define a device\n", devnum);
			goto EXIT;
		}
		
		PRINT("set devnum %d \n", devnum);
		comport=XSPEC_rs232_open(devnum);
		if(comport==0)
		{
			PRINT("Cannot open devnum %d\n", devnum);
			goto EXIT;
		}
		int temp_int_time=XSPEC_rs232_getIntegrationTime(devnum);
		PRINT("Integ time = %d\n",  temp_int_time);
		if(temp_int_time<0)
			goto EXIT;
		XSPEC_rs232_get_SpecConfig0123(devnum, (char *)&cal_coef, 2);
		PRINT("Got coeff \n");
		for(int j=0;j<average;j++)
		{
			XSPEC_rs232_measure_partial(devnum, data_array, 0x1234, temp_int_time, rs_start, rs_end);
			PRINT("Read %d/%d\n", j, average);
					
			process_data(j);
		}
		XSPEC_rs232_close(devnum);
		print_data();

		measure_partial=false;
	}

EXIT:
//	if(comport)XSPEC_rs232_close(comport);
	return 0;
}

void printusage()
{
	printf("RS232 XSPEC_CONTROL usage:\n");
//	printf("-v: verbose\n");
	printf("-d #: com port number. ttyS0=0, ttyS1=1..ttyUSB0=16, ttyUSB1=17..ttyAMA0=22 check rs232 for details\n");
	printf("-M #: mode 0 normal software trigger, 1 rising ext HW, 2 falling ext HW trigger.\n");
	printf("-a #: firmware averaging.\n");	
	printf("-i: get integration time\n");
	printf("-I #: Set Integration Time in RAM\n");
	printf("-L #: Set Integration Time in FLASH\n");
	printf("-c: Dump Config data\n");
	printf("-m: Get a measurement. \n");
	printf("-n #: read # time and take average.\n");
	printf("-p: Get a partial measurement. \n");
	printf("-s #: start pixel 0-3647\n");
	printf("-e #: start pixel 0-3647\n");
}

void PRINT(char *format, ...)
{
//	if(verbose==false)
//		return;
	va_list ap;
	va_start (ap, format);
	vprintf(format, ap);
	va_end (ap);
	fflush(stdout);
}
void print_data()
{
	double dark_value=0;
	double wave[4000];
	int i;
	for(i=cal_coef.dark_start;i<cal_coef.dark_end;i++)
		dark_value+=data_avg[i];
	dark_value=dark_value/(cal_coef.dark_end-cal_coef.dark_start);
	PRINT("dark value %lf\n", dark_value);
	PRINT("cal coef %lE %lE %lE %lE\n", cal_coef.user_coef[0], cal_coef.user_coef[1],cal_coef.user_coef[2],cal_coef.user_coef[3]);
	for(i=0;i<4000;i++)
	{
		wave[i]=cal_coef.user_coef[0]+cal_coef.user_coef[1]*i;
		wave[i]+=cal_coef.user_coef[2]*i*i+cal_coef.user_coef[3]*i*i*i;
	}
	PRINT("pixel    wavelength   value   dark \n"); 
	for(i=cal_coef.data_start;i<=cal_coef.data_start+rs_end-rs_start;i++)
	{	
		int pix;
		pix=i-cal_coef.data_start+rs_start;
		PRINT("%d %lf %lf %lf\n", pix, wave[pix], data_avg[i], dark_value);
	}
}
void process_data(int avg)
{
	int i;
	for(i=0;i<4000;i++)
	{
		double temp;
		temp=data_array[i];
		data_avg[i]=(data_avg[i]*avg+temp)/(avg+1);
	}
}

void PRINT(const char *format, ...)
{
	if(verbose==false)
		return;
	va_list ap;
	va_start (ap, format);
	vprintf(format, ap);
	va_end (ap);
}